@@ -43,18 +43,24 @@ Walkthrough <tutorial-gitfs>`.
 
 # Import python libs
 import copy
+import contextlib
 import distutils.version  # pylint: disable=E0611
+import fcntl
 import glob
 import hashlib
 import logging
 import os
 import re
 import shutil
+import stat
 import subprocess
 from datetime import datetime
+from salt._compat import text_type as _text_type
+from salt._compat import StringIO
 
 VALID_PROVIDERS = ('gitpython', 'pygit2', 'dulwich')
 PER_REMOTE_PARAMS = ('base', 'mountpoint', 'root')
+SYMLINK_RECURSE_DEPTH = 100
 
 # Auth support (auth params can be global or per-remote, too)
 AUTH_PROVIDERS = ('pygit2',)
@@ -166,8 +172,8 @@ def _verify_gitpython(quiet=False):
 
 def _verify_pygit2(quiet=False):
     '''
-    Check if pygit2/libgit2 are available and at a compatible version. Both
-    must be at least 0.21.0.
+    Check if pygit2/libgit2 are available and at a compatible version. Pygit2
+    must be at least 0.20.3 and libgit2 must be at least 0.20.0.
     '''
     if not HAS_PYGIT2:
         log.error(
@@ -225,14 +231,36 @@ def _verify_dulwich(quiet=False):
     '''
     if not HAS_DULWICH:
         log.error(
-            'Git fileserver backend is enabled in master config file, but '
-            'could not be loaded, is Dulwich installed?'
+            'Git fileserver backend is enabled in the master config file, but '
+            'could not be loaded. Is Dulwich installed?'
         )
         if HAS_GITPYTHON and not quiet:
             log.error(_RECOMMEND_GITPYTHON)
         if HAS_PYGIT2 and not quiet:
             log.error(_RECOMMEND_PYGIT2)
         return False
+
+    dulwich_version = dulwich.__version__
+    dulwich_min_version = (0, 9, 4)
+
+    errors = []
+
+    if dulwich_version < dulwich_min_version:
+        errors.append(
+            'Git fileserver backend is enabled in the master config file, but '
+            'the installed version of Dulwich is earlier than {0}. Version {1} '
+            'detected.'.format(dulwich_min_version, dulwich_version)
+        )
+
+        if HAS_PYGIT2 and not quiet:
+            errors.append(_RECOMMEND_PYGIT2)
+        if HAS_GITPYTHON and not quiet:
+            errors.append(_RECOMMEND_GITPYTHON)
+
+        for error in errors:
+            log.error(error)
+        return False
+
     log.info('dulwich gitfs_provider enabled')
     __opts__['verified_gitfs_provider'] = 'dulwich'
     return True
@@ -417,7 +445,7 @@ def _get_tree_dulwich(repo, tgt_env):
                 return repo['repo'].get_object(commit.tree)
 
     # Branch or tag not matched, check if 'tgt_env' is a commit. This is more
-    # difficult with Dulwich because of its inability to deal with tgt_envened
+    # difficult with Dulwich because of its inability to deal with shortened
     # SHA-1 hashes.
     if not _env_is_exposed(tgt_env):
         return None
@@ -614,14 +642,18 @@ def init():
 
     per_remote_defaults = {}
     for param in override_params:
-        per_remote_defaults[param] = __opts__['gitfs_{0}'.format(param)]
+        per_remote_defaults[param] = \
+            _text_type(__opts__['gitfs_{0}'.format(param)])
 
     for remote in __opts__['gitfs_remotes']:
         repo_conf = copy.deepcopy(per_remote_defaults)
         bad_per_remote_conf = False
         if isinstance(remote, dict):
             repo_url = next(iter(remote))
-            per_remote_conf = salt.utils.repack_dictlist(remote[repo_url])
+            per_remote_conf = dict(
+                [(key, _text_type(val)) for key, val in
+                 salt.utils.repack_dictlist(remote[repo_url]).items()]
+            )
             if not per_remote_conf:
                 log.error(
                     'Invalid per-remote configuration for remote {0}. If no '
@@ -856,7 +888,8 @@ def purge_cache():
         remove_dirs = []
     for repo in init():
         try:
-            remove_dirs.remove(repo['hash'])
+            with _aquire_update_lock_for_repo(repo):
+                remove_dirs.remove(repo['hash'])
         except ValueError:
             pass
     remove_dirs = [os.path.join(bp_, rdir) for rdir in remove_dirs
@@ -868,6 +901,37 @@ def purge_cache():
     return False
 
 
+@contextlib.contextmanager
+def _aquire_update_lock_for_repo(repo):
+    provider = _get_provider()
+
+    if provider == 'gitpython':
+        working_dir = repo['repo'].working_dir
+    elif provider == 'pygit2':
+        working_dir = repo['repo'].workdir
+    elif provider == 'dulwich':
+        working_dir = repo['repo'].path
+
+    with wait_for_write_lock(os.path.join(working_dir, 'update.lk')):
+        yield
+
+
+@contextlib.contextmanager
+def wait_for_write_lock(filename):
+    fhandle = open(filename, 'w')
+
+    if salt.utils.is_fcntl_available(check_sunos=True):
+        fcntl.flock(fhandle.fileno(), fcntl.LOCK_EX)
+    try:
+        yield
+    finally:
+        if salt.utils.is_fcntl_available(check_sunos=True):
+            fcntl.flock(fhandle.fileno(), fcntl.LOCK_UN)
+
+        fhandle.close()
+        os.remove(filename)
+
+
 def update():
     '''
     Execute a git fetch on all of the repos
@@ -889,98 +953,93 @@ def update():
             # origin is just a url here, there is no origin object
             origin = repo['url']
             working_dir = repo['repo'].path
-        lk_fn = os.path.join(working_dir, 'update.lk')
-        with salt.utils.fopen(lk_fn, 'w+') as fp_:
-            fp_.write(str(pid))
-        try:
-            log.debug('Fetching from {0}'.format(repo['url']))
-            if provider == 'gitpython':
-                try:
-                    fetch_results = origin.fetch()
-                except AssertionError:
-                    fetch_results = origin.fetch()
-                for fetch in fetch_results:
-                    if fetch.old_commit is not None:
-                        data['changed'] = True
-            elif provider == 'pygit2':
-                try:
-                    origin.credentials = repo['credentials']
-                except KeyError:
-                    # No credentials configured for this repo
-                    pass
-                fetch = origin.fetch()
-                try:
-                    # pygit2.Remote.fetch() returns a dict in pygit2 < 0.21.0
-                    received_objects = fetch['received_objects']
-                except (AttributeError, TypeError):
-                    # pygit2.Remote.fetch() returns a class instance in
-                    # pygit2 >= 0.21.0
-                    received_objects = fetch.received_objects
-                log.debug(
-                    'Gitfs received {0} objects for remote {1}'
-                    .format(received_objects, repo['url'])
-                )
-                if received_objects:
-                    data['changed'] = True
-            elif provider == 'dulwich':
-                client, path = \
-                    dulwich.client.get_transport_and_path_from_url(
-                        origin, thin_packs=True
-                    )
-                refs_pre = repo['repo'].get_refs()
-                try:
-                    refs_post = client.fetch(path, repo['repo'])
-                except dulwich.errors.NotGitRepository:
-                    log.critical(
-                        'Dulwich does not recognize remote {0} as a valid '
-                        'remote URL. Perhaps it is missing \'.git\' at the '
-                        'end.'.format(repo['url'])
-                    )
-                    continue
-                except KeyError:
-                    log.critical(
-                        'Local repository cachedir {0!r} (corresponding '
-                        'remote: {1}) has been corrupted. Salt will now '
-                        'attempt to remove the local checkout to allow it to '
-                        'be re-initialized in the next fileserver cache '
-                        'update.'
-                        .format(repo['cachedir'], repo['url'])
-                    )
+
+        with _aquire_update_lock_for_repo(repo):
+            try:
+                log.debug('Fetching from {0}'.format(repo['url']))
+                if provider == 'gitpython':
                     try:
-                        salt.utils.rm_rf(repo['cachedir'])
-                    except OSError as exc:
-                        log.critical(
-                            'Unable to remove {0!r}: {1}'
-                            .format(repo['cachedir'], exc)
-                        )
-                    continue
-                if refs_post is None:
-                    # Empty repository
-                    log.warning(
-                        'Gitfs remote {0!r} is an empty repository and will '
-                        'be skipped.'.format(origin)
+                        fetch_results = origin.fetch()
+                    except AssertionError:
+                        fetch_results = origin.fetch()
+                    for fetch in fetch_results:
+                        if fetch.old_commit is not None:
+                            data['changed'] = True
+                elif provider == 'pygit2':
+                    try:
+                        origin.credentials = repo['credentials']
+                    except KeyError:
+                        # No credentials configured for this repo
+                        pass
+                    fetch = origin.fetch()
+                    try:
+                        # pygit2.Remote.fetch() returns a dict in pygit2 < 0.21.0
+                        received_objects = fetch['received_objects']
+                    except (AttributeError, TypeError):
+                        # pygit2.Remote.fetch() returns a class instance in
+                        # pygit2 >= 0.21.0
+                        received_objects = fetch.received_objects
+                    log.debug(
+                        'Gitfs received {0} objects for remote {1}'
+                        .format(received_objects, repo['url'])
                     )
-                    continue
-                if refs_pre != refs_post:
-                    data['changed'] = True
-                    # Update local refs
-                    for ref in _dulwich_env_refs(refs_post):
-                        repo['repo'][ref] = refs_post[ref]
-                    # Prune stale refs
-                    for ref in repo['repo'].get_refs():
-                        if ref not in refs_post:
-                            del repo['repo'][ref]
-        except Exception as exc:
-            # Do not use {0!r} in the error message, as exc is not a string
-            log.error(
-                'Exception \'{0}\' caught while fetching gitfs remote {1}'
-                .format(exc, repo['url']),
-                exc_info_on_loglevel=logging.DEBUG
-            )
-        try:
-            os.remove(lk_fn)
-        except (IOError, OSError):
-            pass
+                    if received_objects:
+                        data['changed'] = True
+                elif provider == 'dulwich':
+                    client, path = \
+                        dulwich.client.get_transport_and_path_from_url(
+                            origin, thin_packs=True
+                        )
+                    refs_pre = repo['repo'].get_refs()
+                    try:
+                        refs_post = client.fetch(path, repo['repo'])
+                    except dulwich.errors.NotGitRepository:
+                        log.critical(
+                            'Dulwich does not recognize remote {0} as a valid '
+                            'remote URL. Perhaps it is missing \'.git\' at the '
+                            'end.'.format(repo['url'])
+                        )
+                        continue
+                    except KeyError:
+                        log.critical(
+                            'Local repository cachedir {0!r} (corresponding '
+                            'remote: {1}) has been corrupted. Salt will now '
+                            'attempt to remove the local checkout to allow it to '
+                            'be re-initialized in the next fileserver cache '
+                            'update.'
+                            .format(repo['cachedir'], repo['url'])
+                        )
+                        try:
+                            salt.utils.rm_rf(repo['cachedir'])
+                        except OSError as exc:
+                            log.critical(
+                                'Unable to remove {0!r}: {1}'
+                                .format(repo['cachedir'], exc)
+                            )
+                        continue
+                    if refs_post is None:
+                        # Empty repository
+                        log.warning(
+                            'Gitfs remote {0!r} is an empty repository and will '
+                            'be skipped.'.format(origin)
+                        )
+                        continue
+                    if refs_pre != refs_post:
+                        data['changed'] = True
+                        # Update local refs
+                        for ref in _dulwich_env_refs(refs_post):
+                            repo['repo'][ref] = refs_post[ref]
+                        # Prune stale refs
+                        for ref in repo['repo'].get_refs():
+                            if ref not in refs_post:
+                                del repo['repo'][ref]
+            except Exception as exc:
+                # Do not use {0!r} in the error message, as exc is not a string
+                log.error(
+                    'Exception \'{0}\' caught while fetching gitfs remote {1}'
+                    .format(exc, repo['url']),
+                    exc_info_on_loglevel=logging.DEBUG
+                )
 
     env_cache = os.path.join(__opts__['cachedir'], 'gitfs/envs.p')
     if data.get('changed', False) is True or not os.path.isfile(env_cache):
@@ -1141,98 +1200,157 @@ def find_file(path, tgt_env='base', **kwargs):  # pylint: disable=W0613
                          '{0}.lk'.format(path))
     destdir = os.path.dirname(dest)
     hashdir = os.path.dirname(blobshadest)
-    if not os.path.isdir(destdir):
-        try:
-            os.makedirs(destdir)
-        except OSError:
-            # Path exists and is a file, remove it and retry
-            os.remove(destdir)
-            os.makedirs(destdir)
-    if not os.path.isdir(hashdir):
-        try:
-            os.makedirs(hashdir)
-        except OSError:
-            # Path exists and is a file, remove it and retry
-            os.remove(hashdir)
-            os.makedirs(hashdir)
 
     for repo in init():
-        if repo['mountpoint'] \
-                and not path.startswith(repo['mountpoint'] + os.path.sep):
-            continue
-        repo_path = path[len(repo['mountpoint']):].lstrip(os.path.sep)
-        if repo['root']:
-            repo_path = os.path.join(repo['root'], repo_path)
+        with _aquire_update_lock_for_repo(repo):
+            if not os.path.isdir(destdir):
+                try:
+                    os.makedirs(destdir)
+                except OSError:
+                    # Path exists and is a file, remove it and retry
+                    os.remove(destdir)
+                    os.makedirs(destdir)
+            if not os.path.isdir(hashdir):
+                try:
+                    os.makedirs(hashdir)
+                except OSError:
+                    # Path exists and is a file, remove it and retry
+                    os.remove(hashdir)
+                    os.makedirs(hashdir)
 
-        if provider == 'gitpython':
-            tree = _get_tree_gitpython(repo, tgt_env)
-            if not tree:
-                # Branch/tag/SHA not found in repo, try the next
+            if repo['mountpoint'] \
+                    and not path.startswith(repo['mountpoint'] + os.path.sep):
                 continue
-            try:
-                blob = tree / repo_path
-            except KeyError:
-                continue
-            blob_hexsha = blob.hexsha
+            repo_path = path[len(repo['mountpoint']):].lstrip(os.path.sep)
+            if repo['root']:
+                repo_path = os.path.join(repo['root'], repo_path)
 
-        elif provider == 'pygit2':
-            tree = _get_tree_pygit2(repo, tgt_env)
-            if not tree:
-                # Branch/tag/SHA not found in repo, try the next
-                continue
-            try:
-                oid = tree[repo_path].oid
-                blob = repo['repo'][oid]
-            except KeyError:
-                continue
-            blob_hexsha = blob.hex
-
-        elif provider == 'dulwich':
-            prefix_dirs, _, filename = repo_path.rpartition(os.path.sep)
-            tree = _get_tree_dulwich(repo, tgt_env)
-            tree = _dulwich_walk_tree(repo['repo'], tree, prefix_dirs)
-            if not isinstance(tree, dulwich.objects.Tree):
-                # Branch/tag/SHA not found in repo, try the next
-                continue
-            try:
-                # Referencing the path in the tree returns a tuple, the
-                # second element of which is the object ID of the blob
-                blob = repo['repo'].get_object(tree[filename][1])
-            except KeyError:
-                continue
-            blob_hexsha = blob.sha().hexdigest()
-
-        salt.fileserver.wait_lock(lk_fn, dest)
-        if os.path.isfile(blobshadest) and os.path.isfile(dest):
-            with salt.utils.fopen(blobshadest, 'r') as fp_:
-                sha = fp_.read()
-                if sha == blob_hexsha:
-                    fnd['rel'] = path
-                    fnd['path'] = dest
-                    return fnd
-        with salt.utils.fopen(lk_fn, 'w+') as fp_:
-            fp_.write('')
-        for filename in glob.glob(hashes_glob):
-            try:
-                os.remove(filename)
-            except Exception:
-                pass
-        with salt.utils.fopen(dest, 'w+') as fp_:
+            blob = None
+            depth = 0
             if provider == 'gitpython':
-                blob.stream_data(fp_)
+                tree = _get_tree_gitpython(repo, tgt_env)
+                if not tree:
+                    # Branch/tag/SHA not found in repo, try the next
+                    continue
+                while True:
+                    depth += 1
+                    if depth > SYMLINK_RECURSE_DEPTH:
+                        break
+                    try:
+                        file_blob = tree / repo_path
+                        if stat.S_ISLNK(file_blob.mode):
+                            # Path is a symlink. The blob data corresponding to
+                            # this path's object ID will be the target of the
+                            # symlink. Follow the symlink and set repo_path to the
+                            # location indicated in the blob data.
+                            stream = StringIO()
+                            file_blob.stream_data(stream)
+                            stream.seek(0)
+                            link_tgt = stream.read()
+                            stream.close()
+                            repo_path = os.path.normpath(
+                                os.path.join(os.path.dirname(repo_path), link_tgt)
+                            )
+                        else:
+                            blob = file_blob
+                            break
+                    except KeyError:
+                        # File not found or repo_path points to a directory
+                        break
+                if blob is None:
+                    continue
+                blob_hexsha = blob.hexsha
+
             elif provider == 'pygit2':
-                fp_.write(blob.data)
+                tree = _get_tree_pygit2(repo, tgt_env)
+                if not tree:
+                    # Branch/tag/SHA not found in repo, try the next
+                    continue
+                while True:
+                    depth += 1
+                    if depth > SYMLINK_RECURSE_DEPTH:
+                        break
+                    try:
+                        if stat.S_ISLNK(tree[repo_path].filemode):
+                            # Path is a symlink. The blob data corresponding to this
+                            # path's object ID will be the target of the symlink. Follow
+                            # the symlink and set repo_path to the location indicated
+                            # in the blob data.
+                            link_tgt = repo['repo'][tree[repo_path].oid].data
+                            repo_path = os.path.normpath(
+                                os.path.join(os.path.dirname(repo_path), link_tgt)
+                            )
+                        else:
+                            oid = tree[repo_path].oid
+                            blob = repo['repo'][oid]
+                    except KeyError:
+                        break
+                if blob is None:
+                    continue
+                blob_hexsha = blob.hex
+
             elif provider == 'dulwich':
-                fp_.write(blob.as_raw_string())
-        with salt.utils.fopen(blobshadest, 'w+') as fp_:
-            fp_.write(blob_hexsha)
-        try:
-            os.remove(lk_fn)
-        except (OSError, IOError):
-            pass
-        fnd['rel'] = path
-        fnd['path'] = dest
-        return fnd
+                while True:
+                    depth += 1
+                    if depth > SYMLINK_RECURSE_DEPTH:
+                        break
+                    prefix_dirs, _, filename = repo_path.rpartition(os.path.sep)
+                    tree = _get_tree_dulwich(repo, tgt_env)
+                    tree = _dulwich_walk_tree(repo['repo'], tree, prefix_dirs)
+                    if not isinstance(tree, dulwich.objects.Tree):
+                        # Branch/tag/SHA not found in repo
+                        break
+                    try:
+                        mode, oid = tree[filename]
+                        if stat.S_ISLNK(mode):
+                            # Path is a symlink. The blob data corresponding to
+                            # this path's object ID will be the target of the
+                            # symlink. Follow the symlink and set repo_path to the
+                            # location indicated in the blob data.
+                            link_tgt = repo['repo'].get_object(oid).as_raw_string()
+                            repo_path = os.path.normpath(
+                                os.path.join(os.path.dirname(repo_path), link_tgt)
+                            )
+                        else:
+                            blob = repo['repo'].get_object(oid)
+                            break
+                    except KeyError:
+                        break
+                if blob is None:
+                    continue
+                blob_hexsha = blob.sha().hexdigest()
+
+            salt.fileserver.wait_lock(lk_fn, dest)
+            if os.path.isfile(blobshadest) and os.path.isfile(dest):
+                with salt.utils.fopen(blobshadest, 'r') as fp_:
+                    sha = fp_.read()
+                    if sha == blob_hexsha:
+                        fnd['rel'] = path
+                        fnd['path'] = dest
+                        return fnd
+            with salt.utils.fopen(lk_fn, 'w+') as fp_:
+                fp_.write('')
+            for filename in glob.glob(hashes_glob):
+                try:
+                    os.remove(filename)
+                except Exception:
+                    pass
+            with salt.utils.fopen(dest, 'w+') as fp_:
+                if provider == 'gitpython':
+                    blob.stream_data(fp_)
+                elif provider == 'pygit2':
+                    fp_.write(blob.data)
+                elif provider == 'dulwich':
+                    fp_.write(blob.as_raw_string())
+            with salt.utils.fopen(blobshadest, 'w+') as fp_:
+                fp_.write(blob_hexsha)
+            try:
+                os.remove(lk_fn)
+            except (OSError, IOError):
+                pass
+            fnd['rel'] = path
+            fnd['path'] = dest
+            return fnd
     return fnd
 
 
@@ -1253,10 +1371,9 @@ def serve_file(load, fnd):
     required_load_keys = set(['path', 'loc', 'saltenv'])
     if not all(x in load for x in required_load_keys):
         log.debug(
-            'Not all of the required key in load are present. Missing: {0}'.format(
-                ', '.join(
-                    required_load_keys.difference(load.keys())
-                )
+            'Not all of the required keys present in payload. '
+            'Missing: {0}'.format(
+                ', '.join(required_load_keys.difference(load))
             )
         )
         return ret
@@ -1344,12 +1461,15 @@ def _file_lists(load, form):
         return cache_match
     if refresh_cache:
         ret = {}
-        ret['files'] = _get_file_list(load)
+        ret['files'], ret['symlinks'] = _get_file_list(load)
         ret['dirs'] = _get_dir_list(load)
         if save_cache:
             salt.fileserver.write_file_list_cache(
                 __opts__, ret, list_cache, w_lock
             )
+        # NOTE: symlinks are organized in a dict instead of a list, however the
+        # 'symlinks' key will be defined above so it will never get to the
+        # default value in the call to ret.get() below.
         return ret.get(form, [])
     # Shouldn't get here, but if we do, this prevents a TypeError
     return []
@@ -1378,48 +1498,60 @@ def _get_file_list(load):
 
     provider = _get_provider()
     if 'saltenv' not in load or load['saltenv'] not in envs():
-        return []
-    ret = set()
+        return [], {}
+    files = set()
+    symlinks = {}
     for repo in init():
+        fl_func = None
         if provider == 'gitpython':
-            ret.update(
-                _file_list_gitpython(repo, load['saltenv'])
-            )
+            fl_func = _file_list_gitpython
         elif provider == 'pygit2':
-            ret.update(
-                _file_list_pygit2(repo, load['saltenv'])
-            )
+            fl_func = _file_list_pygit2
         elif provider == 'dulwich':
-            ret.update(
-                _file_list_dulwich(repo, load['saltenv'])
-            )
-    return sorted(ret)
+            fl_func = _file_list_dulwich
+        try:
+            repo_files, repo_symlinks = fl_func(repo, load['saltenv'])
+        except TypeError:
+            # We should never get here unless the gitfs_provider is not
+            # accounted for in tbe above if/elif block.
+            continue
+        else:
+            files.update(repo_files)
+            symlinks.update(repo_symlinks)
+    return sorted(files), symlinks
 
 
 def _file_list_gitpython(repo, tgt_env):
     '''
     Get file list using GitPython
     '''
-    ret = set()
+    files = set()
+    symlinks = {}
     if tgt_env == 'base':
         tgt_env = repo['base']
     tree = _get_tree_gitpython(repo, tgt_env)
     if not tree:
-        return ret
+        return files, symlinks
     if repo['root']:
         try:
             tree = tree / repo['root']
         except KeyError:
-            return ret
-    for blob in tree.traverse():
-        if not isinstance(blob, git.Blob):
+            return files, symlinks
+    relpath = lambda path: os.path.relpath(path, repo['root'])
+    add_mountpoint = lambda path: os.path.join(repo['mountpoint'], path)
+    for file_blob in tree.traverse():
+        if not isinstance(file_blob, git.Blob):
             continue
-        if repo['root']:
-            path = os.path.relpath(blob.path, repo['root'])
-        else:
-            path = blob.path
-        ret.add(os.path.join(repo['mountpoint'], path))
-    return ret
+        file_path = add_mountpoint(relpath(file_blob.path))
+        files.add(file_path)
+        if stat.S_ISLNK(file_blob.mode):
+            stream = StringIO()
+            file_blob.stream_data(stream)
+            stream.seek(0)
+            link_tgt = stream.read()
+            stream.close()
+            symlinks[file_path] = link_tgt
+    return files, symlinks
 
 
 def _file_list_pygit2(repo, tgt_env):
@@ -1429,23 +1561,28 @@ def _file_list_pygit2(repo, tgt_env):
     def _traverse(tree, repo_obj, blobs, prefix):
         '''
         Traverse through a pygit2 Tree object recursively, accumulating all the
-        blob paths within it in the "blobs" list
+        file paths and symlink info in the "blobs" dict
         '''
         for entry in iter(tree):
-            blob = repo_obj[entry.oid]
-            if isinstance(blob, pygit2.Blob):
-                blobs.append(os.path.join(prefix, entry.name))
-            elif isinstance(blob, pygit2.Tree):
-                _traverse(blob,
+            obj = repo_obj[entry.oid]
+            if isinstance(obj, pygit2.Blob):
+                repo_path = os.path.join(prefix, entry.name)
+                blobs.setdefault('files', []).append(repo_path)
+                if stat.S_ISLNK(tree[entry.name].filemode):
+                    link_tgt = repo_obj[tree[entry.name].oid].data
+                    blobs.setdefault('symlinks', {})[repo_path] = link_tgt
+            elif isinstance(obj, pygit2.Tree):
+                _traverse(obj,
                           repo_obj,
                           blobs,
                           os.path.join(prefix, entry.name))
-    ret = set()
+    files = set()
+    symlinks = {}
     if tgt_env == 'base':
         tgt_env = repo['base']
     tree = _get_tree_pygit2(repo, tgt_env)
     if not tree:
-        return ret
+        return files, symlinks
     if repo['root']:
         try:
             # This might need to be changed to account for a root that
@@ -1453,17 +1590,19 @@ def _file_list_pygit2(repo, tgt_env):
             oid = tree[repo['root']].oid
             tree = repo['repo'][oid]
         except KeyError:
-            return ret
+            return files, symlinks
         if not isinstance(tree, pygit2.Tree):
-            return ret
-    blobs = []
+            return files, symlinks
+    blobs = {}
     if len(tree):
         _traverse(tree, repo['repo'], blobs, repo['root'])
-    for blob in blobs:
-        if repo['root']:
-            blob = os.path.relpath(blob, repo['root'])
-        ret.add(os.path.join(repo['mountpoint'], blob))
-    return ret
+    relpath = lambda path: os.path.relpath(path, repo['root'])
+    add_mountpoint = lambda path: os.path.join(repo['mountpoint'], path)
+    for repo_path in blobs.get('files', []):
+        files.add(add_mountpoint(relpath(repo_path)))
+    for repo_path, link_tgt in blobs.get('symlinks', {}).iteritems():
+        symlinks[add_mountpoint(relpath(repo_path))] = link_tgt
+    return files, symlinks
 
 
 def _file_list_dulwich(repo, tgt_env):
@@ -1473,32 +1612,40 @@ def _file_list_dulwich(repo, tgt_env):
     def _traverse(tree, repo_obj, blobs, prefix):
         '''
         Traverse through a dulwich Tree object recursively, accumulating all the
-        blob paths within it in the "blobs" list
+        file paths and symlink info in the "blobs" dict
         '''
         for item in tree.items():
             obj = repo_obj.get_object(item.sha)
             if isinstance(obj, dulwich.objects.Blob):
-                blobs.append(os.path.join(prefix, item.path))
+                repo_path = os.path.join(prefix, item.path)
+                blobs.setdefault('files', []).append(repo_path)
+                mode, oid = tree[item.path]
+                if stat.S_ISLNK(mode):
+                    link_tgt = repo_obj.get_object(oid).as_raw_string()
+                    blobs.setdefault('symlinks', {})[repo_path] = link_tgt
             elif isinstance(obj, dulwich.objects.Tree):
                 _traverse(obj,
                           repo_obj,
                           blobs,
                           os.path.join(prefix, item.path))
-    ret = set()
+    files = set()
+    symlinks = {}
     if tgt_env == 'base':
         tgt_env = repo['base']
     tree = _get_tree_dulwich(repo, tgt_env)
     tree = _dulwich_walk_tree(repo['repo'], tree, repo['root'])
     if not isinstance(tree, dulwich.objects.Tree):
-        return ret
-    blobs = []
+        return files, symlinks
+    blobs = {}
     if len(tree):
         _traverse(tree, repo['repo'], blobs, repo['root'])
-    for blob in blobs:
-        if repo['root']:
-            blob = os.path.relpath(blob, repo['root'])
-        ret.add(os.path.join(repo['mountpoint'], blob))
-    return ret
+    relpath = lambda path: os.path.relpath(path, repo['root'])
+    add_mountpoint = lambda path: os.path.join(repo['mountpoint'], path)
+    for repo_path in blobs.get('files', []):
+        files.add(add_mountpoint(relpath(repo_path)))
+    for repo_path, link_tgt in blobs.get('symlinks', {}).iteritems():
+        symlinks[add_mountpoint(relpath(repo_path))] = link_tgt
+    return files, symlinks
 
 
 def file_list_emptydirs(load):  # pylint: disable=W0613
@@ -1651,3 +1798,27 @@ def _dir_list_dulwich(repo, tgt_env):
             blob = os.path.relpath(blob, repo['root'])
         ret.add(os.path.join(repo['mountpoint'], blob))
     return ret
+
+
+def symlink_list(load):
+    '''
+    Return a dict of all symlinks based on a given path in the repo
+    '''
+    if 'env' in load:
+        salt.utils.warn_until(
+            'Boron',
+            'Passing a salt environment should be done using \'saltenv\' '
+            'not \'env\'. This functionality will be removed in Salt Boron.'
+        )
+        load['saltenv'] = load.pop('env')
+
+    if load['saltenv'] not in envs():
+        return {}
+    try:
+        prefix = load['prefix'].strip('/')
+    except KeyError:
+        prefix = ''
+    symlinks = _file_lists(load, 'symlinks')
+    return dict([(key, val)
+                 for key, val in symlinks.iteritems()
+                 if key.startswith(prefix)])
