@@ -10,13 +10,6 @@ option.
 import os
 import logging
 
-try:
-    import fcntl
-    HAS_FCNTL = os.uname()[0] != "SunOS"
-except ImportError:
-    # fcntl is not available on windows
-    HAS_FCNTL = False
-
 # Import salt libs
 import salt.fileserver
 import salt.utils
@@ -38,6 +31,7 @@ def find_file(path, saltenv='base', env=None, **kwargs):
         # Backwards compatibility
         saltenv = env
 
+    path = os.path.normpath(path)
     fnd = {'path': '',
            'rel': ''}
     if os.path.isabs(path):
@@ -94,8 +88,7 @@ def serve_file(load, fnd):
         return ret
     ret['dest'] = fnd['rel']
     gzip = load.get('gzip', None)
-
-    with salt.utils.fopen(fnd['path'], 'rb') as fp_:
+    with salt.utils.fopen(os.path.normpath(fnd['path']), 'rb') as fp_:
         fp_.seek(load['loc'])
         data = fp_.read(__opts__['file_buffer_size'])
         if gzip and data:
@@ -128,8 +121,13 @@ def update():
     if os.path.exists(mtime_map_path):
         with salt.utils.fopen(mtime_map_path, 'rb') as fp_:
             for line in fp_:
-                file_path, mtime = line.split(':', 1)
-                old_mtime_map[file_path] = mtime
+                try:
+                    file_path, mtime = line.split(':', 1)
+                    old_mtime_map[file_path] = mtime
+                except ValueError:
+                    # Document the invalid entry in the log
+                    log.warning('Skipped invalid cache mtime entry in {0}: {1}'
+                                .format(mtime_map_path, line))
 
     # generate the new map
     new_mtime_map = salt.fileserver.generate_mtime_map(__opts__['file_roots'])
@@ -148,7 +146,12 @@ def update():
 
     if __opts__.get('fileserver_events', False):
         # if there is a change, fire an event
-        event = salt.utils.event.MasterEvent(__opts__['sock_dir'])
+        event = salt.utils.event.get_event(
+                'master',
+                __opts__['sock_dir'],
+                __opts__['transport'],
+                opts=__opts__,
+                listen=False)
         event.fire_event(data, tagify(['roots', 'update'], prefix='fileserver'))
 
 
@@ -217,15 +220,10 @@ def file_hash(load, fnd):
     if not os.path.exists(cache_dir):
         os.makedirs(cache_dir)
     # save the cache object "hash:mtime"
-    if HAS_FCNTL:
-        with salt.utils.flopen(cache_path, 'w') as fp_:
-            fp_.write('{0}:{1}'.format(ret['hsum'], os.path.getmtime(path)))
-            fcntl.flock(fp_.fileno(), fcntl.LOCK_UN)
-        return ret
-    else:
-        with salt.utils.fopen(cache_path, 'w') as fp_:
-            fp_.write('{0}:{1}'.format(ret['hsum'], os.path.getmtime(path)))
-        return ret
+    cache_object = '{0}:{1}'.format(ret['hsum'], os.path.getmtime(path))
+    with salt.utils.flopen(cache_path, 'w') as fp_:
+        fp_.write(cache_object)
+    return ret
 
 
 def _file_lists(load, form):
@@ -269,6 +267,8 @@ def _file_lists(load, form):
                     path,
                     followlinks=__opts__['fileserver_followsymlinks']):
                 dir_rel_fn = os.path.relpath(root, path)
+                if __opts__.get('file_client', 'remote') == 'local' and os.path.sep == "\\":
+                    dir_rel_fn = dir_rel_fn.replace('\\', '/')
                 ret['dirs'].append(dir_rel_fn)
                 if len(dirs) == 0 and len(files) == 0:
                     if not salt.fileserver.is_file_ignored(__opts__, dir_rel_fn):
@@ -284,6 +284,8 @@ def _file_lists(load, form):
                                 path
                             )
                     if not salt.fileserver.is_file_ignored(__opts__, rel_fn):
+                        if __opts__.get('file_client', 'remote') == 'local' and os.path.sep == "\\":
+                            rel_fn = rel_fn.replace('\\', '/')
                         ret['files'].append(rel_fn)
         if save_cache:
             salt.fileserver.write_file_list_cache(
