@@ -18,6 +18,7 @@ from collections import MutableMapping
 from salt.exceptions import LoaderError
 from salt.template import check_render_pipe_str
 from salt.utils.decorators import Depends
+from salt.utils.odict import OrderedDict
 
 # Solve the Chicken and egg problem where grains need to run before any
 # of the modules are loaded and are generally available for any usage.
@@ -124,6 +125,8 @@ def minion_mods(opts, context=None, whitelist=None):
         import salt.loader
 
         __opts__ = salt.config.minion_config('/etc/salt/minion')
+        __grains__ = salt.loader.grains(__opts__)
+        __opts__['grains'] = __grains__
         __salt__ = salt.loader.minion_mods(__opts__)
         __salt__['test.ping']()
     '''
@@ -714,7 +717,7 @@ class Loader(object):
         '''
         Return a dict of functions found in the defined module_dirs
         '''
-        funcs = {}
+        funcs = OrderedDict()
         self.load_modules()
         for mod in self.modules:
             # If this is a proxy minion then MOST modules cannot work.  Therefore, require that
@@ -824,7 +827,7 @@ class Loader(object):
         self.modules = []
 
         log.trace('loading {0} in {1}'.format(self.tag, self.module_dirs))
-        names = {}
+        names = OrderedDict()
         disable = set(self.opts.get('disable_{0}s'.format(self.tag), []))
 
         cython_enabled = False
@@ -1238,15 +1241,6 @@ class Loader(object):
         grains_data = {}
         funcs = self.gen_functions()
         for key, fun in funcs.items():
-            if not key.startswith('core.'):
-                continue
-            ret = fun()
-            if not isinstance(ret, dict):
-                continue
-            grains_data.update(ret)
-        for key, fun in funcs.items():
-            if key.startswith('core.'):
-                continue
             try:
                 ret = fun()
             except Exception:
@@ -1260,7 +1254,10 @@ class Loader(object):
                 continue
             if not isinstance(ret, dict):
                 continue
-            grains_data.update(ret)
+            # The OrderedDict of funcs has core grains at the end. So earlier
+            # grains are the ones which should override
+            ret.update(grains_data)
+            grains_data = ret
         # Write cache if enabled
         if self.opts.get('grains_cache', False):
             cumask = os.umask(077)
