@@ -25,26 +25,26 @@ def __virtual__():
     '''
     Only work on systems that have been booted with systemd
     '''
-    if __grains__['kernel'] == 'Linux' and _sd_booted():
+    if __grains__['kernel'] == 'Linux' and _sd_booted(__context__):
         return __virtualname__
     return False
 
 
-def _sd_booted():
+def _sd_booted(context):
     '''
     Return True if the system was booted with systemd, False otherwise.
     '''
     # We can cache this for as long as the minion runs.
-    if "systemd.sd_booted" not in __context__:
+    if "systemd.sd_booted" not in context:
         try:
             # This check does the same as sd_booted() from libsystemd-daemon:
             # http://www.freedesktop.org/software/systemd/man/sd_booted.html
             if os.stat('/run/systemd/system'):
-                __context__['systemd.sd_booted'] = True
+                context['systemd.sd_booted'] = True
         except OSError:
-            __context__['systemd.sd_booted'] = False
+            context['systemd.sd_booted'] = False
 
-    return __context__['systemd.sd_booted']
+    return context['systemd.sd_booted']
 
 
 def _canonical_unit_name(name):
@@ -377,7 +377,8 @@ def _templated_instance_enabled(name):
 
 def _enabled(name):
     is_enabled = \
-        not __salt__['cmd.retcode'](_systemctl_cmd('is-enabled', name))
+        not __salt__['cmd.retcode'](_systemctl_cmd('is-enabled', name),
+                                    ignore_retcode=True)
     return is_enabled or _templated_instance_enabled(name)
 
 
