@@ -15,27 +15,23 @@ to use a YAML 'explicit key', as demonstrated in the second example below.
 .. code-block:: yaml
 
     AAAAB3NzaC1kc3MAAACBAL0sQ9fJ5bYTEyY==:
-      ssh_auth:
-        - present
+      ssh_auth.present:
         - user: root
         - enc: ssh-dss
 
     ? AAAAB3NzaC1kc3MAAACBAL0sQ9fJ5bYTEyY==...
     :
-      ssh_auth:
-        - present
+      ssh_auth.present:
         - user: root
         - enc: ssh-dss
 
     thatch:
-      ssh_auth:
-        - present
+      ssh_auth.present:
         - user: root
         - source: salt://ssh_keys/thatch.id_rsa.pub
 
     sshkeys:
-      ssh_auth:
-        - present
+      ssh_auth.present:
         - user: root
         - enc: ssh-rsa
         - options:
@@ -173,11 +169,31 @@ def present(
         return ret
 
     if source != '':
-        data = __salt__['ssh.set_auth_key_from_file'](
-                user,
+        key = __salt__['cp.get_file_str'](
                 source,
-                config,
                 saltenv=__env__)
+        fileHasOptions = False
+        # check if this is of form {options} {enc} {key} {comment}
+        sshre = re.compile(r'^(ssh\-|ecds).*')
+        key = key.rstrip().split('\n')
+        for keyline in key:
+            fileHasOptions = sshre.match(keyline)
+            if not fileHasOptions:
+                data = __salt__['ssh.set_auth_key_from_file'](
+                        user,
+                        source,
+                        config,
+                        saltenv=__env__)
+            else:
+                # Split keyline to get key und commen
+                keyline = keyline.split(' ')
+                data = __salt__['ssh.set_auth_key'](
+                        user,
+                        keyline[1],
+                        keyline[0],
+                        keyline[2],
+                        options or [],
+                        config)
     else:
         # check if this is of form {options} {enc} {key} {comment}
         sshre = re.compile(r'^(.*?)\s?((?:ssh\-|ecds)[\w-]+\s.+)$')
